function [heat_transfer_coeff, flue_gas_state_in, air_state_out] = ...
        CalcLuvoHeatTransferCoefficient(....
            flue_gas_state_out,...
            air_state_compressed,...
            phi,...
            species,...
            SMediaData)
    [mu, Cpdot_min, flue_gas_state_in, air_state_out] = IterateMu(flue_gas_state_out, air_state_compressed, phi, species, SMediaData);
    heat_transfer_coeff = Cpdot_min ./ (mu-1) .* log((phi-1)./(phi.*mu-1));
end

function [mu, Cpdot_min, flue_gas_state_in, air_state_out] =...
    IterateMu(...
            flue_gas_state_out,...
            air_state_in,...
            phi,...
            species,...
            SMediaData)
    flue_gas_state_in = flue_gas_state_out;
    air_state_out = air_state_in;
    Cpdot_min = 0;
    Cpdot_min_prev = 100;
    while abs(Cpdot_min - Cpdot_min_prev) > 1e-4
        Cpdot_min_prev = Cpdot_min;
        [mu, Cpdot_min, flue_gas_state_in, air_state_out] = CalcMu(...
            flue_gas_state_out,...
            flue_gas_state_in,...
            air_state_in,...
            air_state_out,...
            phi,...
            species,...
            SMediaData);
    end
end

function [mu, Cpdot_min, flue_gas_state_in, air_state_out] = ...
        CalcMu(...
            flue_gas_state_out,...
            flue_gas_state_in,...
            air_state_in,...
            air_state_out,...
            phi,...
            species,...
            SMediaData)
    if flue_gas_state_out.T == flue_gas_state_in.T
        Cpdot_flue_gas = flue_gas_state_out.Cpdot;
    else
        Cpdot_flue_gas = (flue_gas_state_out.Hdot - flue_gas_state_in.Hdot)/(flue_gas_state_out.T - flue_gas_state_in.T);
    end
    if air_state_out.T == air_state_in.T
        Cpdot_air = air_state_out.Cpdot;
    else
        Cpdot_air = (air_state_out.Hdot - air_state_in.Hdot)/(air_state_out.T - air_state_in.T);
    end
    if Cpdot_flue_gas < Cpdot_air
        mu = Cpdot_flue_gas / Cpdot_air;
        Cpdot_min = Cpdot_flue_gas;
        flue_gas_state_in = th_TpState(...
            'T', (phi*air_state_in.T-flue_gas_state_out.T)/(phi-1),...
            'p', flue_gas_state_out.p,...
            'psi', flue_gas_state_out.psi,...
            'ndot', flue_gas_state_out.ndot,...
            'species', species,...
            'gas_eos_method', 1,...
            'MediaData', SMediaData);
        air_state_out = th_HpState(...
            'Hdot', air_state_in.Hdot - (flue_gas_state_out.Hdot - flue_gas_state_in.Hdot),...
            'p', air_state_in.p,...
            'T_initial', air_state_out.T,...
            'psi', air_state_in.psi,...
            'ndot', air_state_in.ndot,...
            'species', species,...
            'gas_eos_method', 1,...
            'MediaData', SMediaData);
    else
        mu = Cpdot_air / Cpdot_flue_gas;
        Cpdot_min = Cpdot_air;
        flue_gas_state_in = th_TpState(...
            'T', (phi*mu*air_state_in.T-flue_gas_state_out.T)/(phi*mu-1),...
            'p', flue_gas_state_out.p,...
            'psi', flue_gas_state_out.psi,...
            'ndot', flue_gas_state_out.ndot,...
            'species', species,...
            'gas_eos_method', 1,...
            'MediaData', SMediaData);
        air_state_out = th_HpState(...
            'Hdot', air_state_in.Hdot - (flue_gas_state_out.Hdot - flue_gas_state_in.Hdot),...
            'p', air_state_in.p,...
            'T_initial', air_state_out.T,...
            'psi', air_state_in.psi,...
            'ndot', air_state_in.ndot,...
            'species', species,...
            'gas_eos_method', 1,...
            'MediaData', SMediaData);
    end
end