function [heat_transfer_coeff, state2_out] = CalcSizeHeatTransferCoefficient(state1_in, state1_out, state2_in, species, SMediaData)
    Qdot = state1_out.Hdot - state1_in.Hdot;
    state2_out = th_HpState(...
            'Hdot', state2_in.Hdot - Qdot,...
            'p', state2_in.p,...
            'T_initial', state1_out.T,...
            'psi', state2_in.psi,...
            'ndot', state2_in.ndot,...
            'species', species,...
            'gas_eos_method', 1,...
            'MediaData', SMediaData);
    if state1_in.T == state1_out.T
        Cpdot1 = state1_in.Cpdot;
    else
        Cpdot1 = -Qdot/(state1_in.T - state1_out.T);
    end
    if state2_in.T == state2_out.T
        Cpdot2 = state2_in.Cpdot;
    else
        Cpdot2 = Qdot/(state2_in.T - state2_out.T);
    end
    if state1_in.x ~= state1_out.x
        mu = 0;
        Cpdot_min = Cpdot2;
    elseif state2_in.x ~= state2_out.x
        mu = 0;
        Cpdot_min = Cpdot1;
    else
        if Cpdot1 < Cpdot2
            mu = Cpdot1 / Cpdot2;
            Cpdot_min = Cpdot1;
        else
            mu = Cpdot2 / Cpdot1;
            Cpdot_min = Cpdot2;
        end
    end
    phi = abs(Qdot) / (Cpdot_min * abs(state1_in.T - state2_in.T));
    heat_transfer_coeff = Cpdot_min ./ (mu-1) .* log((phi-1)./(phi.*mu-1));
end