th_LoadDefaultMediaData();
species = {'H2O'; 'N2'; 'O2'; 'CH4'; 'CO2'};
psi_water = strcmp(species, 'H2O');
psi_O2 = double(strcmp(species, 'O2'));
psi_N2 = double(strcmp(species, 'N2'));
psi_CO2 = double(strcmp(species, 'CO2'));
psi_CH4 = double(strcmp(species, 'CH4'));
species_full = {SMediaData.SSpecies.Species};
th_M = zeros(size(species));
for i = 1:length(species)
    th_M(i) = SMediaData.SSpecies(strcmp(species_full, species{i})).M;
end

%% Fuel in
fuel_state_in = th_TpState('T', 300,...
    'p', 1e5,...
    'psi', psi_CH4,...
    'ndot', 250,...
    'species', species,...
    'gas_eos_method', 1, ...
    'MediaData', SMediaData);
o_min = 2;
lambda = 1.3;

%% Air in
air_state_in.T = 300;
air_state_in.p = 101325;
p_sat_water = th_SaturationPressure(...
    'T', air_state_in.T,...
    'species', species(psi_water),...
    'MediaData', SMediaData);
relative_humidity_air = 0.6;
water_fraction_air = relative_humidity_air*p_sat_water/air_state_in.p;
air_state_in.psi = (psi_O2*0.21+psi_N2*0.79)*(1-water_fraction_air)+psi_water*water_fraction_air;
l_min = o_min / air_state_in.psi(psi_O2==1);
air_state_in = th_TpState('T', air_state_in.T,...
    'p', air_state_in.p,...
    'psi', air_state_in.psi,...
    'ndot', fuel_state_in.ndot * l_min * lambda,...
    'species', species,...
    'gas_eos_method', 1, ...
    'MediaData', SMediaData);

%% Compression
fuel_state_compressed = th_IsentropicCompression(fuel_state_in, 1.5e5, 0.8, species, SMediaData);
air_state_compressed = th_IsentropicCompression(air_state_in, 1.5e5, 0.8, species, SMediaData);

%% Air Preheater
combustion_state.psi = ...
    psi_CO2...
    + (2 + air_state_in.psi(psi_water) * lambda * l_min) .* psi_water...
    + (lambda-1) * o_min * psi_O2...
    + air_state_in.psi(psi_N2==1) * lambda * l_min * psi_N2;
combustion_state.ndot = sum(combustion_state.psi);
combustion_state.psi = combustion_state.psi / combustion_state.ndot;
combustion_state.ndot = combustion_state.ndot * fuel_state_compressed.ndot;
flue_gas_state_out = th_TpState(...
    'T', 273.15+150,...
    'p', fuel_state_compressed.p,...
    'psi', combustion_state.psi,...
    'ndot', combustion_state.ndot,...
    'species', species,...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);
[luvo_size, flue_gas_state_boiler_out, air_state_preheated] = CalcLuvoHeatTransferCoefficient(flue_gas_state_out, air_state_compressed, 0.6, species, SMediaData);

%% Combustion
combustion_state = th_HpState(...
    'Hdot', air_state_preheated.Hdot + fuel_state_compressed.Hdot,...
    'p', fuel_state_compressed.p,...
    'T_initial', 800,...
    'psi', combustion_state.psi,...
    'ndot', combustion_state.ndot,...
    'species', species,...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);

%% Drum
drum_state_liquid.p = 35e5;
drum_state_liquid = th_TpxState(...
    'T', th_SaturationTemperature(...
        'p', drum_state_liquid.p,...
        'species', species(psi_water),...
        'MediaData', SMediaData),...
    'p', drum_state_liquid.p,...
    'x', 0,...
    'psi', 1,...
    'ndot', 1000/th_M(psi_water),...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);

drum_state_steam = th_TpxState(...
    'T', drum_state_liquid.T,...
    'p', drum_state_liquid.p,...
    'x', 1,...
    'psi', 1,...
    'ndot', 1000/th_M(psi_water),...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);

drum_state_density_liquid = th_Density(...
    'T', drum_state_liquid.T,...
    'p', drum_state_liquid.p,...
    'x', 0,...
    'psi', 1,...
    'ndot', drum_state_liquid.ndot,...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);
drum_state_density_steam = th_Density(...
    'T', drum_state_liquid.T,...
    'p', drum_state_liquid.p,...
    'x', 1,...
    'psi', 1,...
    'ndot', drum_state_liquid.ndot,...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);
x = drum_state_density_steam.rho / (drum_state_density_liquid.rho + drum_state_density_steam.rho);

%% High Pressure Turbine
turbine_hp_state_out.p = 1e5;
turbine_hp_state_out = th_TpxState(...
    'T', th_SaturationTemperature(...
        'p', turbine_hp_state_out.p,...
        'species', species(psi_water),...
        'MediaData', SMediaData),...
    'p', turbine_hp_state_out.p,...
    'x', 0.96,...
    'psi', 1,...
    'ndot', drum_state_steam.ndot,...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);

turbine_hp_state_in = th_IsentropicExpansion(turbine_hp_state_out, drum_state_steam.p, 0.85, species(psi_water), SMediaData);

%% Low Pressure Turbine
turbine_lp_state_out.p = 1e3;
turbine_lp_state_out = th_TpxState(...
    'T', th_SaturationTemperature(...
        'p', turbine_lp_state_out.p,...
        'species', species(psi_water),...
        'MediaData', SMediaData),...
    'p', turbine_lp_state_out.p,...
    'x', 0.96,...
    'psi', 1,...
    'ndot', drum_state_steam.ndot,...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);

turbine_lp_state_in = th_IsentropicExpansion(turbine_lp_state_out, turbine_hp_state_out.p, 0.85, species(psi_water), SMediaData);

%% Feed Water
feed_water_state = th_TpxState(...
    'T', th_SaturationTemperature(...
        'p', turbine_lp_state_out.p,...
        'species', species(psi_water),...
        'MediaData', SMediaData),...
    'p', turbine_lp_state_out.p,...
    'x', 0,...
    'psi', 1,...
    'ndot', drum_state_steam.ndot,...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);

%% Feed Water Pump
feed_water_state_density = th_Density(...
    'T', feed_water_state.T,...
    'p', feed_water_state.p,...
    'psi', 1,...
    'ndot', drum_state_steam.ndot,...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);
feed_water_state_pumped = th_HpState(...
    'Hdot', feed_water_state.Hdot + (drum_state_liquid.p - feed_water_state.p) * 1000 / feed_water_state_density.rho / 0.8,...
    'p', drum_state_liquid.p,...
    'T_initial', feed_water_state.T,...
    'psi', feed_water_state.psi,...
    'ndot', feed_water_state.ndot,...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);

%% Mass Flow
water_mass_flow = (combustion_state.Hdot - flue_gas_state_boiler_out.Hdot) / (turbine_hp_state_in.Hdot - feed_water_state_pumped.Hdot + turbine_lp_state_in.Hdot - turbine_hp_state_out.Hdot);
drum_state_steam = th_TpxState(...
    'T', drum_state_steam.T,...
    'p', drum_state_steam.p,...
    'x', drum_state_steam.x,...
    'psi', 1,...
    'ndot', drum_state_steam.ndot * water_mass_flow,...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);
drum_state_liquid = th_TpxState(...
    'T', drum_state_liquid.T,...
    'p', drum_state_liquid.p,...
    'x', drum_state_liquid.x,...
    'psi', 1,...
    'ndot', drum_state_liquid.ndot * water_mass_flow,...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);
turbine_hp_state_in = th_TpxState(...
    'T', turbine_hp_state_in.T,...
    'p', turbine_hp_state_in.p,...
    'x', turbine_hp_state_in.x,...
    'psi', 1,...
    'ndot', turbine_hp_state_in.ndot * water_mass_flow,...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);
turbine_hp_state_out = th_TpxState(...
    'T', turbine_hp_state_out.T,...
    'p', turbine_hp_state_out.p,...
    'x', turbine_hp_state_out.x,...
    'psi', 1,...
    'ndot', turbine_hp_state_out.ndot * water_mass_flow,...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);
turbine_lp_state_in = th_TpxState(...
    'T', turbine_lp_state_in.T,...
    'p', turbine_lp_state_in.p,...
    'x', turbine_lp_state_in.x,...
    'psi', 1,...
    'ndot', turbine_lp_state_in.ndot * water_mass_flow,...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);
turbine_lp_state_out = th_TpxState(...
    'T', turbine_lp_state_out.T,...
    'p', turbine_lp_state_out.p,...
    'x', turbine_lp_state_out.x,...
    'psi', 1,...
    'ndot', turbine_lp_state_out.ndot * water_mass_flow,...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);
feed_water_state = th_TpxState(...
    'T', feed_water_state.T,...
    'p', feed_water_state.p,...
    'x', feed_water_state.x,...
    'psi', 1,...
    'ndot', feed_water_state.ndot * water_mass_flow,...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);
feed_water_state_pumped = th_TpxState(...
    'T', feed_water_state_pumped.T,...
    'p', feed_water_state_pumped.p,...
    'x', feed_water_state_pumped.x,...
    'psi', 1,...
    'ndot', feed_water_state_pumped.ndot * water_mass_flow,...
    'species', species(psi_water),...
    'gas_eos_method', 1,...
    'MediaData', SMediaData);

%% Superheater
[superheater_size, flue_gas_state_superheater_out] = CalcSizeHeatTransferCoefficient(drum_state_steam, turbine_hp_state_in, combustion_state, species, SMediaData);

%% Evaporator
[evaporator_size, flue_gas_state_evaporator_out] = CalcSizeHeatTransferCoefficient(drum_state_liquid, drum_state_steam, flue_gas_state_superheater_out, species, SMediaData);

%% Reheater
[reheater_size, flue_gas_state_reheater_out] = CalcSizeHeatTransferCoefficient(turbine_hp_state_out, turbine_lp_state_in, flue_gas_state_evaporator_out, species, SMediaData);

%% Economizer
[economizer_size, flue_gas_state_economizer_out] = CalcSizeHeatTransferCoefficient(feed_water_state_pumped, drum_state_liquid, flue_gas_state_reheater_out, species, SMediaData);

%% Power
power_net = turbine_hp_state_in.Hdot - turbine_hp_state_out.Hdot + turbine_lp_state_in.Hdot - turbine_lp_state_out.Hdot;
power_net = power_net - (fuel_state_compressed.Hdot - fuel_state_in.Hdot) - (air_state_compressed.Hdot - air_state_in.Hdot) - (feed_water_state_pumped.Hdot - feed_water_state.Hdot);

%% Heating Value
fuel_air_mixture_state.psi = (fuel_state_in.ndot * fuel_state_in.psi...
    + air_state_preheated.ndot * air_state_preheated.psi)...
    / (fuel_state_in.ndot + air_state_preheated.ndot);
fuel_air_mixture_state = th_TpState(...
    'T', 298,...
    'p', fuel_state_compressed.p,...
    'psi', fuel_air_mixture_state.psi,...
    'ndot', (fuel_state_in.ndot + air_state_preheated.ndot),...
    'species', species,...
    'gas_eos_method', 1, ...
    'MediaData', SMediaData);
combustion_state_enthalpy_of_formation = th_TpState(...
    'T', 298,...
    'p', combustion_state.p,...
    'psi', combustion_state.psi,...
    'ndot', combustion_state.ndot,...
    'species', species,...
    'gas_eos_method', 1, ...
    'MediaData', SMediaData);
heating_value = (combustion_state.Hdot - fuel_air_mixture_state.Hdot) / (fuel_state_in.ndot * th_M(psi_CH4==1));
efficiency = power_net/(fuel_state_in.ndot * th_M(psi_CH4==1) * 5e4);